@@ -1,7 +1,8 @@
 #ifndef THREADING_H
 #define THREADING_H
 
-#include <boost/align/aligned_alloc.hpp>
+#include "alloc.hpp"
+#include <atomic>
 
 //mp_limb_t is an unsigned integer
 static_assert(sizeof(mp_limb_t)==8, "");
@@ -156,60 +157,6 @@ static uint64 get_time_cycles() {
     #define TRACK_CYCLES_OUTPUT_STATS
 #endif
 
-//use realloc or free to free the memory
-void* alloc_cache_line(size_t bytes) {
-    //round up to the next multiple of 64
-    size_t aligned_bytes=((bytes+63)>>6)<<6;
-
-    //padding so that the result
-    aligned_bytes+=64;
-
-    void* res=boost::alignment::aligned_alloc(64, aligned_bytes); // aligned_alloc(64, aligned_bytes);
-
-    //the address modulo 64 is used to decide if the gmp integer is allocated on the stack or the heap
-    //stack integers are 64-aligned and heap integers aren't
-    res=((char*)res)+16;
-
-    assert((uint64(res)&63)==16); //must have this alignment for correctness
-    return res;
-}
-
-void free_cache_line(void* ptr) {
-    boost::alignment::aligned_free(((char*)ptr)-16); //free(old_ptr);
-}
-
-void* mp_alloc_func(size_t new_bytes) {
-    return alloc_cache_line(new_bytes);
-}
-
-void mp_free_func(void* old_ptr, size_t old_bytes) {
-    //either mp_alloc_func allocated old_ptr and it is 64-aligned, or it points to data in mpz and its address equals 16 modulo 64
-    assert((uint64(old_ptr)&63)==0 || (uint64(old_ptr)&63)==16);
-
-    if ((uint64(old_ptr)&63)==16) {
-        //mp_alloc_func allocated this, so it can be freed with std::free
-        free_cache_line(old_ptr);
-    } else {
-        //this is part of the mpz struct defined below. it can't be freed, so do nothing
-    }
-}
-
-void* mp_realloc_func(void* old_ptr, size_t old_bytes, size_t new_bytes) {
-    void* res=mp_alloc_func(new_bytes);
-
-    memcpy(res, old_ptr, (old_bytes<new_bytes)? old_bytes : new_bytes);
-
-    mp_free_func(old_ptr, old_bytes);
-
-    return res;
-}
-
-//must call this before calling any gmp functions
-//(the mpz class constructor does not call any gmp functions)
-void init_gmp() {
-    mp_set_memory_functions(mp_alloc_func, mp_realloc_func, mp_free_func);
-}
-
 template<int d_expected_size, int d_padded_size> struct alignas(64) mpz;
 
 template<int expected_size_out, int padded_size_out, int expected_size_a, int padded_size_a, int expected_size_b, int padded_size_b>
@@ -591,35 +538,6 @@ template<int d_expected_size, int d_padded_size> struct alignas(64) mpz {
     }
 };
 
-template<class type> struct cache_line_ptr {
-    type* ptr=nullptr;
-
-    cache_line_ptr() {}
-    cache_line_ptr(cache_line_ptr& t)=delete;
-    cache_line_ptr(cache_line_ptr&& t) { swap(ptr, t.ptr); }
-
-    cache_line_ptr& operator=(cache_line_ptr& t)=delete;
-    cache_line_ptr& operator=(cache_line_ptr&& t) { swap(ptr, t.ptr); }
-
-    ~cache_line_ptr() {
-        if (ptr) {
-            ptr->~type();
-            boost::alignment::aligned_free(ptr); // wjb free(ptr);
-            ptr=nullptr;
-        }
-    }
-
-    type& operator*() const { return *ptr; }
-    type* operator->() const { return ptr; }
-};
-
-template<class type, class... arg_types> cache_line_ptr<type> make_cache_line(arg_types&&... args) {
-    cache_line_ptr<type> res;
-    res.ptr=(type*)alloc_cache_line(sizeof(type));
-    new(res.ptr) type(forward<arg_types>(args)...);
-    return res;
-}
-
 template<bool is_write, class type> void prefetch(const type& p) {
     //write prefetching lowers performance but read prefetching increases it
     if (is_write) return;
@@ -632,19 +550,13 @@ template<bool is_write, class type> void prefetch(const type& p) {
 template<class type> void prefetch_write(const type& p) { prefetch<true>(p); }
 template<class type> void prefetch_read(const type& p) { prefetch<false>(p); }
 
-void memory_barrier() {
-    asm volatile( "" ::: "memory" );
-}
-
 struct alignas(64) thread_counter {
-    uint64 counter_value=0; //updated atomically since only one thread can write to it
-    uint64 error_flag=0;
+    std::atomic<uint64> counter_value{0}; //updated atomically since only one thread can write to it
+    std::atomic<bool> error_flag{false};
 
     void reset() {
-        memory_barrier();
         counter_value=0;
-        error_flag=0;
-        memory_barrier();
+        error_flag = false;
     }
 
     thread_counter() {
@@ -680,10 +592,8 @@ struct thread_state {
             //print( "raise_error", is_slave );
         //}
 
-        memory_barrier();
-        this_counter().error_flag=1;
-        other_counter().error_flag=1;
-        memory_barrier();
+        this_counter().error_flag = true;
+        other_counter().error_flag = true;
     }
 
     uint64 v() {
@@ -697,11 +607,9 @@ struct thread_state {
             return true;
         }
 
-        memory_barrier();
-
         uint64 spin_counter=0;
         while (other_counter().counter_value < t_v) {
-            if (this_counter().error_flag || other_counter().error_flag) {
+            if (this_counter().error_flag.load() || other_counter().error_flag.load()) {
                 raise_error();
                 break;
             }
@@ -716,11 +624,8 @@ struct thread_state {
             }
 
             ++spin_counter;
-            memory_barrier();
         }
 
-        memory_barrier();
-
         if (!(this_counter().error_flag)) {
             last_fence=t_v;
         }
@@ -739,8 +644,6 @@ struct thread_state {
             return true;
         }
 
-        memory_barrier(); //wait for all writes to finish (on x86 this doesn't do anything but the compiler still needs it)
-
         assert(t_v>=v());
 
         if (this_counter().error_flag) {
@@ -749,7 +652,6 @@ struct thread_state {
 
         this_counter().counter_value=t_v;
 
-        memory_barrier(); //want the counter writes to be low latency so prevent the compiler from caching it
         return !(this_counter().error_flag);
     }
 
@@ -764,17 +666,15 @@ struct thread_state {
     /*void wait_for_error_to_be_cleared() {
         assert(is_slave && enable_threads);
         while (this_counter().error_flag) {
-            memory_barrier();
+            std::this_thread::yield();
         }
     }
 
     void clear_error() {
         assert(!is_slave);
 
-        memory_barrier();
-        this_counter().error_flag=0;
-        other_counter().error_flag=0;
-        memory_barrier();
+        this_counter().error_flag = false;
+        other_counter().error_flag = false;
     }*/
 };
 
@@ -879,7 +779,8 @@ template<class mpz_type> bool gcd_unsigned(
     data.threshold=(uint64*)&threshold[0];
 
     data.uv_counter_start=c_thread_state.counter_start+counter_start_delta+1;
-    data.out_uv_counter_addr=&(c_thread_state.this_counter().counter_value);
+    // TODO: come up with something better here
+    data.out_uv_counter_addr=reinterpret_cast<uint64_t*>(&(c_thread_state.this_counter().counter_value));
     data.out_uv_addr=(uint64*)&(c_results.uv_entries[1]);
     data.iter=-1;
     data.a_end_index=(a_limbs==0)? 0 : a_limbs-1;
@@ -888,12 +789,9 @@ template<class mpz_type> bool gcd_unsigned(
         assert((uint64(data.out_uv_addr)&63)==0); //should be cache line aligned
     }
 
-    memory_barrier();
     int error_code=hasAVX2()?
-	    asm_code::asm_avx2_func_gcd_unsigned(&data):
-	    asm_code::asm_cel_func_gcd_unsigned(&data);
-
-    memory_barrier();
+        asm_code::asm_avx2_func_gcd_unsigned(&data):
+        asm_code::asm_cel_func_gcd_unsigned(&data);
 
     if (error_code!=0) {
         c_thread_state.raise_error();
